/*
 * RTCRV3029.c
 *
 * Created: 12.08.2017 05:02:02
 *  Author: Nenninger
 */ 

#include "i2c.h"
#include "RTCRV3029.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>

/*
 * transform decimal value to bcd
 */
uint8_t RTC_dec2bcd(uint8_t val) {
	return val + 6 * (val / 10);
}

/*
 * transform bcd value to decimal
 */
uint8_t RTC_bcd2dec(uint8_t val) {
	return val - 6 * (val >> 4);
}

void RTC_GetTime(uint8_t *hours, uint8_t *minutes, uint8_t *seconds){
	*hours = RTC_bcd2dec(I2C_ReadRegister(RTCAdr,HOURS_REGISTER));
	*minutes = RTC_bcd2dec(I2C_ReadRegister(RTCAdr,MINUTES_REGISTER));
	*seconds = RTC_bcd2dec(I2C_ReadRegister(RTCAdr,SECONDS_REGISTER));
	/*if (*hours & 0x40) // 12hr mode:
		*hours &= 0x1F; // use bottom 5 bits (pm bit = temp & 0x20)
	else *hours &= 0x3F; // 24hr mode: use bottom 6 bits*/
}

void RTC_GetDate(uint8_t *months, uint8_t *days, uint8_t *years){
	*months = RTC_bcd2dec(I2C_ReadRegister(RTCAdr,MONTHS_REGISTER));
	*days = RTC_bcd2dec(I2C_ReadRegister(RTCAdr,DAYS_REGISTER));
	*years = RTC_bcd2dec(I2C_ReadRegister(RTCAdr,YEARS_REGISTER));
}

void RTC_GetTemp(uint8_t *temp){
	*temp = I2C_ReadRegister(RTCAdr,TEMPERATURE_REGISTER);
}

void RTC_SetDateTimeDate(uint8_t *days, uint8_t *months, uint8_t *years, uint8_t *hours, uint8_t *minutes, uint8_t *seconds){
	/*uint8_t MyByte = I2C_ReadRegister(RTCAdr,CONTROL_1);
	MyByte &= ~(1 << 0);
	I2C_WriteRegister(RTCAdr,CONTROL_1,MyByte);
	*/
	
	I2C_WriteRegister(RTCAdr,MONTHS_REGISTER, RTC_dec2bcd(*months));
	I2C_WriteRegister(RTCAdr,DAYS_REGISTER, RTC_dec2bcd(*days));
	I2C_WriteRegister(RTCAdr,YEARS_REGISTER, RTC_dec2bcd(*years));
	I2C_WriteRegister(RTCAdr,HOURS_REGISTER, RTC_dec2bcd(*hours));
	I2C_WriteRegister(RTCAdr,MINUTES_REGISTER,RTC_dec2bcd(*minutes));
	I2C_WriteRegister(RTCAdr,SECONDS_REGISTER, RTC_dec2bcd(*seconds));
	
	/*MyByte |= (1 << 0);
	I2C_WriteRegister(RTCAdr,CONTROL_1,MyByte);*/
}


void RTC_UrInit(void){ //1 Hz out Tempcontroled
// 	uint8_t tmp;
// 	tmp = I2C_ReadRegister(RTCAdr,CONTROL_REGISTER);
// 	tmp |= (1<<2) | (1<<4); //Bit 3+4
//  I2C_WriteRegister(RTCAdr,CONTROL_REGISTER,tmp);
	I2C_WriteRegister(RTCAdr,CONTROL_REGISTER,0b00001110);
}

void RTC_INIT(void){
	uint8_t MyByte = I2C_ReadRegister(RTCAdr,CONTROL_STATUS);
	if((MyByte & (1<<RTC_PON)) | (MyByte & (1<<RTC_SR))){
		I2C_WriteRegister(RTCAdr,CONTROL_STATUS,0);
	}
}

void RTC_START(void){
	//I2C_WriteRegister(RTCAdr,CONTROL_RESET,0b00010000);
	//RTC_INIT();
	uint8_t MyByte = I2C_ReadRegister(RTCAdr,CONTROL_1);
	MyByte |= (1 << 0);
	I2C_WriteRegister(RTCAdr,CONTROL_1,MyByte);
}

void RTC_STOP(void){
	//I2C_WriteRegister(RTCAdr,CONTROL_RESET,0b00010000);
	uint8_t MyByte = I2C_ReadRegister(RTCAdr,CONTROL_1);
	MyByte &= ~(1 << 0);
	I2C_WriteRegister(RTCAdr,CONTROL_1,MyByte);
	
}