/*
 * NTStartUhr2.c
 *
 * Created: 12.08.2017 04:32:16
 * Author : Nenninger
 */ 

#define F_CPU 14745600UL
#define CTC_MATCH_OVERFLOW ((F_CPU / 1000u) / 8u)

#define sbi(ADDRESS,BIT) (ADDRESS |= (1LL<<BIT))	//setze BIT
#define lbi(ADDRESS,BIT) (ADDRESS &= ~(1LL<<BIT))	//lsche BIT
#define tbi(ADDRESS,BIT) (ADDRESS ^= (1LL<<BIT))	// toggle BIT


#include <avr/io.h>
#include <time.h>
#include <util/eu_dst.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>

#include "i2c.h"
#include "RTCRV3029.h"
#include "BH1750.h"

#include "global.h"

#include "WSMatrixSenkrecht.h"
#include "SiebenSegment.h"

//GPS String
#define GPS_char_timeout 2
#define GPS_char_count 100
volatile uint8_t GPS_string[GPS_char_count]="";
volatile uint8_t GPS_string_pos = 0;
volatile uint8_t GPS_waitchar = 0;

uint16_t MilliSekunden;
uint16_t TonAnZeit;
uint16_t RTCNullZeit=0;

#define HELLIGKEIT_MIN 20
#define HELLIGKEIT_MAX 235
uint16_t lux=0xffff;
uint16_t lux_alt=0xffff;
uint8_t helligkeit=HELLIGKEIT_MIN;

volatile uint16_t SekundenBisStart = 0;

struct tm time_tm;
time_t temp_time;

uint8_t aktSekunde;

volatile struct {
	unsigned GPSabg:1;
	unsigned GPSData:1;
	unsigned EineSekunde:1; 
	unsigned halbeSekunde:1; 
	unsigned mitStartnummer:1;
	unsigned StartZeit:3; //0= aus 1/2 Wechsel 3=festeZeit 4=Drehschalter 7=zur vollen Minute
}status;

void wait(int ms){
	for(int i=0;i<ms;i++){
		_delay_ms(1);
	}
}

void LEDSetzeHintergrund(uint8_t rot,uint8_t gruen,uint8_t blau){
	Flaeche(0,0,39,32,rot,gruen,blau);
}

void TonAn(uint8_t start,uint16_t zeit){
	
	TCNT2 = 0; //wichtig da sonst evtuele zu hoch!!!
	if(start==0){
		OCR2A = 0xff;
	}else{
		OCR2A = 0x70;
	}
	TonAnZeit=zeit;
	
}

void TonAus(void){
	OCR2A = 0;
	TCCR2B |= (1<<FOC2A);
	
}

//GPS UART0
#define BAUDGPS 57600UL

// Berechnungen Baudrate
#define UBRR_VALGPS ((F_CPU+BAUDGPS*8)/(BAUDGPS*16)-1)   // clever runden
#define BAUD_REALGPS (F_CPU/(16*(UBRR_VALGPS+1)))     // Reale Baudrate
#define BAUD_ERRORGPS ((BAUD_REALGPS*1000)/BAUDGPS) // Fehler in Promille, 1000 = kein Fehler.

#if ((BAUD_ERRORGPS<990) || (BAUD_ERRORGPS>1010))
#error Systematischer Fehler der Baudrate GPS!!!!
#endif

//UART0 (GPS) initialisieren
void UART_INIT_GPS(void){
	UCSR0B = (1<<TXEN0)|(1<<RXEN0)|(1<<RXCIE0);
	UCSR0C = (1<<UCSZ01) | (1<<UCSZ00);
	
	UBRR0H = UBRR_VALGPS >> 8;
	UBRR0L = UBRR_VALGPS & 0xFF;
}

void DEBUG_SendByte(byte data){
	loop_until_bit_is_set(UCSR0A,UDRE0);	/* warten bis Senden moeglich */
	UDR0 = data;								/* sende Zeichen */
}


//RS485 UART1
#define BAUDRS485 57600UL

// Berechnungen Baudrate
#define UBRR_VALRS485 ((F_CPU+BAUDRS485*8)/(BAUDRS485*16)-1)   // clever runden
#define BAUD_REALRS485 (F_CPU/(16*(UBRR_VALRS485+1)))     // Reale Baudrate
#define BAUD_ERRORRS485 ((BAUD_REALRS485*1000)/BAUDRS485) // Fehler in Promille, 1000 = kein Fehler.

#if ((BAUD_ERRORRS485<990) || (BAUD_ERRORRS485>1010))
#error Systematischer Fehler der Baudrate RS485!!!!
#endif

//UART1 (RS485) initialisieren
void UART_INIT_RS485(void){
	UCSR1B = (1<<TXEN1)|(1<<RXEN1)|(1<<RXCIE1);
	UCSR1C = (1<<UCSZ11) | (1<<UCSZ10);
	
	UBRR1H = UBRR_VALRS485 >> 8;
	UBRR1L = UBRR_VALRS485 & 0xFF;
}

//RS485 Byte senden
void RS485_SendByte(byte data){
	loop_until_bit_is_set(UCSR1A,UDRE1);	/* warten bis Senden moeglich */
	UDR1 = data;								/* sende Zeichen */
}

uint8_t holeKodierschalterStellung(uint8_t port){
	uint8_t wert=0;
	lbi(PORTA,port);
	_delay_us(1);
	wert = PINA>>1;
	wert &= 0b00001111;
	wert ^= 0x0f;
	sbi(PORTA,port);
	return wert;	
}

void pruefeEingaenge(void){
	uint8_t stunde,minute,sekunde;
	
	if(!(PINC & (1<<PC3))){ //Start
		if ((!(PINC & (1<<PC7))) && (status.StartZeit == 0)){ //start zur vollen Minute
			
			RTC_GetTime(&stunde,&minute,&sekunde);
			SekundenBisStart = 60 - sekunde;
			status.StartZeit = 7;
			
		}else{
			if (!(PINC & (1<<PC5))){
			
				//Kodierschalter lesen
				SekundenBisStart = holeKodierschalterStellung(PA5) * 100;
				SekundenBisStart += holeKodierschalterStellung(PA6) * 10;
				SekundenBisStart += holeKodierschalterStellung(PA7) * 1;
				status.StartZeit = 4;
			
			}else if (!(PINC & (1<<PC6))){
			
				SekundenBisStart = 15;
				status.StartZeit = 3;
			
			}else{
			
				if(status.StartZeit != 1){
					
					status.StartZeit = 1;
					SekundenBisStart = 3;
					
				}else{
					
					status.StartZeit = 2;
					SekundenBisStart = 27;
					
				}
						
			}
		}
	}else{
		status.StartZeit = 0;
	}
}

void AnzUhrzeit(void){
	uint8_t stunde,minute,sekunde;
	uint8_t x = 1;
	uint8_t y = 0;
	RTC_GetTime(&stunde,&minute,&sekunde);
	aktSekunde = sekunde;
	if((stunde/10) == 1){
		x += 2;
	}
	Out5x7Char((stunde/10)+0x30,x,y,helligkeit,helligkeit,helligkeit);
	stunde -= (stunde/10)*10;
	Out5x7Char((stunde)+0x30,7,y,helligkeit,helligkeit,helligkeit);
	
	Out5x7Char(':',13,y,helligkeit,helligkeit,helligkeit);
	
	x = 15;
	if((minute/10) == 1){
		x += 2;
	}
	Out5x7Char((minute/10)+0x30,x,y,helligkeit,helligkeit,helligkeit);
	minute -= (minute/10)*10;
	Out5x7Char((minute)+0x30,21,y,helligkeit,helligkeit,helligkeit);
	
	Out5x7Char(':',27,y,helligkeit,helligkeit,helligkeit);
	
	x = 29;
	if((sekunde/10) == 1){
		x += 2;
	}
	Out5x7Char((sekunde/10)+0x30,x,y,helligkeit,helligkeit,helligkeit);
	sekunde -= (sekunde/10)*10;
	Out5x7Char((sekunde)+0x30,35,y,helligkeit,helligkeit,helligkeit);
}

void AnzDatum(void){
	uint8_t tag,monat,jahr;
	uint8_t zeile=25;
	uint8_t x = 1;
	RTC_GetDate(&monat,&tag,&jahr);
	if((tag/10) == 1){
		x += 2;
	}
	Out5x7Char((tag/10)+0x30,x,zeile,helligkeit,helligkeit,helligkeit);
	tag -= (tag/10)*10;
	Out5x7Char((tag)+0x30,7,zeile,helligkeit,helligkeit,helligkeit);
	
	Out5x7Char('.',13,zeile,helligkeit,helligkeit,helligkeit);
	
	x = 15;
	if((monat/10) == 1){
		x += 2;
	}
	Out5x7Char((monat/10)+0x30,x,zeile,helligkeit,helligkeit,helligkeit);
	monat -= (monat/10)*10;
	Out5x7Char((monat)+0x30,21,zeile,helligkeit,helligkeit,helligkeit);
	
	Out5x7Char('.',27,zeile,helligkeit,helligkeit,helligkeit);
	
	x = 29;
	if((jahr/10) == 1){
		x += 2;
	}
	Out5x7Char((jahr/10)+0x30,x,zeile,helligkeit,helligkeit,helligkeit);
	jahr -= (jahr/10)*10;
	Out5x7Char((jahr)+0x30,35,zeile,helligkeit,helligkeit,helligkeit);
}

void AnzLogo(void){
	uint8_t zeile=13;
	uint8_t spalte=5;
	spalte += Out5x7Char('N',spalte,zeile,0,0,helligkeit)+1;
	spalte += Out5x7Char('-',spalte,zeile,0,0,helligkeit)+1;
	spalte += Out5x7Char('T',spalte,zeile,0,0,helligkeit)+1;
	spalte += Out5x7Char('o',spalte,zeile,0,0,helligkeit)+1;
	spalte += Out5x7Char('o',spalte,zeile-1,0,0,helligkeit)+1;
	spalte += Out5x7Char('l',spalte,zeile,0,0,helligkeit)+1;
	Out5x7Char('s',spalte,zeile,0,0,helligkeit);
}

void AnzHelligkeit(void){
	uint8_t zeile=17;
	uint8_t spalte=16;
	uint8_t hell = helligkeit;
	spalte += Out5x7Char(hell/100+0x30,spalte,zeile,helligkeit,helligkeit,helligkeit)+1;
	hell -= (hell/100)*100;
	spalte += Out5x7Char(hell/10+0x30,spalte,zeile,helligkeit,helligkeit,helligkeit)+1;
	hell -= (hell/10)*10;
	Out5x7Char(hell+0x30,spalte,zeile,helligkeit,helligkeit,helligkeit);
}

void AnzLogoRST(void){
	uint8_t spalte=0;
	Flaeche(spalte,9,14,24,helligkeit/4,0,0);
	SEGw(spalte+11,9,2,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+10,10,4,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+1,11,2,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+10,11,4,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+1,12,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+10,12,4,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+2,13,11,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+3,14,11,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+8,15,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+12,15,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+7,16,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+12,16,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+6,17,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+12,17,2,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+5,18,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+11,18,2,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+4,19,5,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+10,19,1,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+3,20,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+7,20,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+2,21,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+8,21,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+1,22,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+9,22,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte,23,3,helligkeit,helligkeit,helligkeit);
	SEGw(spalte+10,23,3,helligkeit,helligkeit,helligkeit);
	
	Flaeche(spalte+15,9,39,24,0,0,helligkeit/4);
	
	/*spalteschrift = Out5x7Char('R',spalteschrift+16,zeile,helligkeit,helligkeit,helligkeit)+1;
	spalteschrift += Out5x7Char('S',spalteschrift+16,zeile,helligkeit,helligkeit,helligkeit)+1;
	Out5x7Char('T',spalteschrift+16,zeile,helligkeit,helligkeit,helligkeit)+1;*/
	
	//temp
	char temp = 0;
	uint8_t ttemp = 0;
	RTC_GetTemp(&ttemp);
	temp = ttemp - 0x3C;
	
	if(temp < 0){
		temp=temp*-1;
		Out5x7Char('-',20,10,helligkeit,helligkeit,helligkeit);
	}
	temp = temp - ((temp/100)*100);
	if(( temp/100 > 0) | (temp/10 > 0))
		Out5x7Char((temp/10)+0x30,25,10,helligkeit,helligkeit,helligkeit);
	
	temp -= (temp /10)*10;
		Out5x7Char(temp+0x30,31,10,helligkeit,helligkeit,helligkeit);
	
	SEGw(37,9,2,helligkeit,helligkeit,helligkeit);
	SEGw(37,10,2,helligkeit,helligkeit,helligkeit);
	
	AnzHelligkeit();
}

void AnzSekunden(void){
	volatile uint16_t temp_SekundenBisStart=SekundenBisStart;
		
	if(SekundenBisStart == 0){
		//Schalterabfragen
		pruefeEingaenge();
		//anzSekunden = 105;
		if(status.mitStartnummer)
		  Flaeche(0,18,8,32,0,helligkeit,0);
		else
		  Flaeche(0,7,39,16,0,helligkeit,0);
		TonAn(1,750);
		
	}else{
		if(SekundenBisStart < 5)
			TonAn(0,100);
		if(status.mitStartnummer){
			Flaeche(0,18,8,32,helligkeit,0,0);
		}else{
			Flaeche(33,8,38,15,helligkeit,0,0);
			if(SekundenBisStart > 4)
				Flaeche(1,8,6,15,helligkeit,0,0);
			if(SekundenBisStart > 3)
				Flaeche(9,8,14,15,helligkeit,0,0);
			if(SekundenBisStart > 2)
				Flaeche(17,8,22,15,helligkeit,0,0);
			if(SekundenBisStart > 1)
				Flaeche(25,8,30,15,helligkeit,0,0);
		}
	}
	
	//Ziffern 3.Zeile Sekunden
	if(SekundenBisStart/100 > 0)
		Out10x16Char((SekundenBisStart/100)+0x30,6,16,helligkeit,helligkeit,helligkeit);
	
	temp_SekundenBisStart = SekundenBisStart - ((SekundenBisStart/100)*100);
	if(( SekundenBisStart/100 > 0) | (temp_SekundenBisStart/10 > 0))
		Out10x16Char((temp_SekundenBisStart/10)+0x30,18,16,helligkeit,helligkeit,helligkeit);
	
	temp_SekundenBisStart -= (temp_SekundenBisStart /10)*10;
	Out10x16Char(temp_SekundenBisStart+0x30,30,16,helligkeit,helligkeit,helligkeit);
}

void holeHelligkeit(void){
	uint16_t lux_temp = 0;
	//helligkeit
	if(I2C_Detect(BH1750_ADR)){
		
		lux_temp = BH1750_Read();
		if( ((lux_temp > lux_alt) && ((lux_temp - lux_alt)>20)) || ((lux_temp < lux_alt) && ((lux_alt - lux_temp)>20)) ){
			lux_alt = lux_temp;
			lux = lux_temp;
			lux = (lux>>3);
			
		}
		
		if(lux > 0x00ff){
			helligkeit = 255;
		}else{
			if(helligkeit < lux){
				if((lux - helligkeit > 10) && (helligkeit < HELLIGKEIT_MAX-10) ){
					helligkeit += 10;
				}else{
					helligkeit++;
				}
			}else if(helligkeit > lux){
				if((helligkeit - lux > 10) && (helligkeit > HELLIGKEIT_MIN+10)){
					helligkeit -= 10;
				}else{
					helligkeit--;
				}
			}
		}
			
		//MAX MIN Setzen
		if(helligkeit < HELLIGKEIT_MIN){
			helligkeit = HELLIGKEIT_MIN;
		}else if(helligkeit > HELLIGKEIT_MAX){
			helligkeit = HELLIGKEIT_MAX;
		}
		
	}else{
		SEGw(0,1,1,10,0,0);
		ws2812_setleds(leds,AnzahlLEDs);
	}
}

void init(void){
  //Ausgnge
	//A0 = LEDs Out /A5-A7 = Kodierschalter
	DDRA |= (1<<PA0) | (1<<PA5) | (1<<PA6) | (1<<PA7);
	
	//B0 GPS_sync //B2 Timer 1sekund //B3 LED_Takt_RTC // B4 Error
	DDRB |= (1<<PB0) | (1<<PB2) | (1<<PB3) | (1<<PB4);
	
	//
	//DDRC |= (1<<PC6) | (1<<PC7) ;
		
	//D4 MAX485 //D7 Sound Output
	DDRD |= (1<<PD4) | (1<<PD7);
	
  //Eingnge Pullups
    //PC2 = RTC_Clock
	//PC3 = start
	//PC4 = reset
	//PC5 = Zeit1
	//PC6 = Zeit2
	//PC7 = start volle Sekunde = 1 | volle Minute = 0
	PORTC |= ( (1<<PC3) | (1<<PC4) | (1<<PC5) | (1<<PC6) | (1<<PC7) ); 
	
	//Kodierschalter fr Zeit (1,2,4,8)
	PORTA |= ( (1<<PA1) | (1<<PA2) | (1<<PA3) | (1<<PA4) );
	
  //Interrupt fr RTC_Clock
	//EICRA |= (1<<ISC21) | (1<<ISC20);
	PCICR |= (1<<PCIE2);
	PCMSK2 |= (1<<PCINT18);  //PCINT18 enable RTCAdr_CLK
	
  //Timer0
	
  //Timer1 ~1ms
	TCCR1B |= (1 << WGM12) | (1 << CS11);
	OCR1AH = (CTC_MATCH_OVERFLOW >> 8);
	OCR1AL = CTC_MATCH_OVERFLOW;
	TIMSK1 |= (1 << OCIE1A);
	
	//Timer2 Sound
	TCCR2A = (1<<COM2A0) | (1<<WGM20);  //Waveform generation mode
	TCCR2B = (1<<CS22) | (0<<CS20) | (1<<WGM22);    //Clock select
		
	status.GPSabg = 0;
	status.mitStartnummer = 0;
	status.StartZeit = 0;
	status.halbeSekunde = 0;
	status.EineSekunde = 0;
	
	MilliSekunden=0;
	
	set_zone(1);
	set_dst(eu_dst);
	
	//nun Init ...
	I2C_Init();
	//RTC_UrInit();
	RTC_INIT();
	UART_INIT_GPS();
	UART_INIT_RS485();
	BH1750_Init();
	
	//Ausgnge fr Zeiteingabe auf high!
	sbi(PORTA,PA5);
	sbi(PORTA,PA6);
	sbi(PORTA,PA7);
	
	ACSR = (1<<ACD);
}

//ISRs

//GPS
ISR(USART0_RX_vect){
	
	if(status.GPSabg==0){
		byte zeichen;
		zeichen = UDR0;
		GPS_waitchar = GPS_char_timeout;
		GPS_string[GPS_string_pos] = zeichen;
		if(GPS_string_pos < GPS_char_count){
			GPS_string_pos++;
		}
		if(zeichen == 0x0A){
			status.GPSData = 1;
			
		}
	}
	
}

//RS485
ISR(USART1_RX_vect){
	unsigned char buffer;
	buffer = UDR1;
}

//Timer 1 ca 1ms
ISR(TIMER1_COMPA_vect){
	
	if(MilliSekunden < 1000){
		MilliSekunden++;
	}
	
	if(MilliSekunden == 1000){
		MilliSekunden = 0;
		sbi(PORTB,PB2);
	}else if(MilliSekunden == 500){
		lbi(PORTB,PB2);
	}
	
	//Ton
	if(TonAnZeit==1){
		TonAus();
	}
	
	if(TonAnZeit>0){
		TonAnZeit--;
	}
	
	//GPS
	if(GPS_waitchar>0){
		GPS_waitchar--;
	}
	if((GPS_waitchar==0) & (GPS_string_pos>0)){
		GPS_string_pos = 0;
	}
	
	//RTC genau starten
	if(RTCNullZeit>0){
		RTCNullZeit--;
		if(RTCNullZeit==0){
			RTC_START();
		}
	}
}

 //RTC 1Hz
ISR(PCINT2_vect){
	
	if ((PINC & (1<<PC2))){
		status.EineSekunde = 1;
	}else{
		status.halbeSekunde = 1;
	}
	tbi(PORTB,PB3);
	
}

//nicht abgefangene ISR
ISR(BADISR_vect){
	PORTB |= (1 << PB4);
}

/*uint8_t istSommerzeit(uint8_t hour, uint8_t day, uint8_t wday, uint8_t month){

	if( month < 3 || month > 10 )			// month 1, 2, 11, 12
		return 0;					// -> Winter

	if( day - wday >= 25 && (wday || hour >= 2) ){ // after last Sunday 2:00
		if( month == 10 )				// October -> Winter
			return 0;
	}else{					// before last Sunday 2:00
		if( month == 3 )				// March -> Winter
			return 0;
	}
	return 1;
}*/

int main(void){
    init();
	
	
	//Boot
	sbi(PORTB,PB4);
	LEDSetzeHintergrund(0,0,0);
	AnzLogo();
	ws2812_setleds(leds,AnzahlLEDs);
	wait(750);
	lbi(PORTB,PB4);
	sei();
	wdt_enable(WDTO_2S);
	while (1) {
		wdt_reset();
			
		//reset
		if (!(PINC & (1<<PC4))){
			SekundenBisStart = 0;
			status.StartZeit = 0;
			status.EineSekunde = 1;
		}
		
		if(status.GPSData == 1){
			//$GPRMC,095058.00,A,5008.46295,N,00854.42218,E,0.593,,080917,,,A*7C
			uint8_t pos=18;
			uint8_t GPSStunde,GPSMinute,GPSSekunde;
			uint8_t GPSTag,GPSMonat,GPSJahr;
			uint8_t AnzKommar=0;
			uint8_t PosDatum=0;
			status.GPSData = 0;
			if(GPS_string[0]=='$' && GPS_string[1]=='G' && GPS_string[2]=='P' && GPS_string[3]=='R' && GPS_string[4]=='M' && GPS_string[5]=='C' && GPS_string[17] == 'A'){
				if(RTCNullZeit==0) {
					RTC_STOP();
					RTCNullZeit = 55;
				}
				GPSStunde=(GPS_string[7]-'0')*10;
				GPSStunde+=(GPS_string[8]-'0');
				GPSMinute=(GPS_string[9]-'0')*10;
				GPSMinute+=(GPS_string[10]-'0');
				GPSSekunde=(GPS_string[11]-'0')*10;
				GPSSekunde+=(GPS_string[12]-'0');
				status.GPSabg=1;
				sbi(PORTB,PB0);
				while((GPS_string[pos] != 0x0d) && (pos < GPS_char_count)){
					if(GPS_string[pos]==','){
						AnzKommar++;
						if(AnzKommar==7)
						  PosDatum = pos+1;
					}
					pos++;
				}
				GPSTag=(GPS_string[PosDatum]-'0')*10;
				GPSTag+=(GPS_string[PosDatum+1]-'0');
				GPSMonat=(GPS_string[PosDatum+2]-'0')*10;
				GPSMonat+=(GPS_string[PosDatum+3]-'0');
				GPSJahr=(GPS_string[PosDatum+4]-'0')*10;
				GPSJahr+=(GPS_string[PosDatum+5]-'0');
				time_tm.tm_hour=GPSStunde;
				time_tm.tm_min =GPSMinute;
				time_tm.tm_sec=GPSSekunde;
				time_tm.tm_mday=GPSTag;
				//time_tm.tm_mon = 5;
				time_tm.tm_mon=GPSMonat-1;
				time_tm.tm_year=GPSJahr+100;
				temp_time = mktime(&time_tm);
				temp_time += (ONE_HOUR); //Deutschland
				/*
				if(istSommerzeit(time_tm.tm_hour,time_tm.tm_mday,time_tm.tm_wday,time_tm.tm_mon+1)){
					temp_time += (ONE_HOUR * 2);
				}else{
					temp_time += (ONE_HOUR);
				}
				*/
				//+ Sekunde Ausgleich Laufzeit bis zur Ausgabe
				temp_time += 1;
								
				localtime_r(&temp_time,&time_tm);
				GPSMonat = time_tm.tm_mon +1;
				GPSJahr = time_tm.tm_year -100;
		
				RTC_SetDateTimeDate((uint8_t *) &time_tm.tm_mday,&GPSMonat,&GPSJahr,(uint8_t *) &time_tm.tm_hour,(uint8_t *) &time_tm.tm_min,(uint8_t *) &time_tm.tm_sec);
			}
		}
		
		//jede halbeSekunde
		if(status.halbeSekunde == 1){
			status.halbeSekunde = 0;
			
			/*SEGw(39,7,1,0,0,helligkeit);
			ws2812_setleds(leds,AnzahlLEDs);*/
			//holeHelligkeit();
		}
		
		//jede Sekunde
		if(status.EineSekunde == 1){
			status.EineSekunde = 0;
			
			//Anzeige
			LEDSetzeHintergrund(0,0,0);
			AnzUhrzeit();
			if(SekundenBisStart==0){
				pruefeEingaenge();
				AnzDatum();
				if(aktSekunde<3){
					AnzLogo();
				}else{
					AnzLogoRST();
				}
			}else{
				SekundenBisStart--;
				AnzSekunden();
				
			}
			
			ws2812_setleds(leds,AnzahlLEDs);
			
			holeHelligkeit();
			
		}
		
    }
	return 0;
}

